/*

   All the resources for this project: https://randomnerdtutorials.com/
   Modified by NL
   Created by FILIPEFLOP

*/

#include <SPI.h>
#include <MFRC522.h>
#include "pitches.h"

#define SS_PIN 10   // Configurable, see typical pin layout above
#define RST_PIN 9   // Configurable, see typical pin layout above

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance.

#define Relay  41
#define Taster 1

#define Buzzer 42
#define BUZZER_PWM_CHANNEL 0

#define NO_ACTION         0
#define ACCESS_GRANTED    1
#define ACCESS_DENIDED    2

int SigMod;
long detectionTime;
long setLED;

#define MIN_PAUZA     300
#define SIG_DURATION    3000
#define SET_LED_TIME    250

String content = "";
byte letter;

// notes in the melody:
int AGmelody[] = {
  NOTE_DS5, NOTE_E5, NOTE_F5,
  NOTE_F5, NOTE_FS5, NOTE_G5, NOTE_G5, NOTE_GS5, NOTE_A5, NOTE_B5
};
// note durations: 4 = quarter note, 8 = eighth note, etc.:
int AGnoteDurations[] = {
  32, 32, 32,
  32, 32, 32, 32, 32, 16, 8
};

// notes in the melody:
int ADmelody[] = {
  NOTE_G4, NOTE_C4
};
// note durations: 4 = quarter note, 8 = eighth note, etc.:
int ADnoteDurations[] = {
  4, 4
};

unsigned char notes, currentNote;
int pauseBetweenNotes;
unsigned long lastNote;
int* melody;
int* noteDur;

//Light signalisation
void Signals(int Mod) {
  static int noAct;

  if (Mod == NO_ACTION) {
    if (millis() - setLED > SET_LED_TIME) {
      if (noAct % 5) {
        neopixelWrite(RGB_BUILTIN,0,0,0); // All OFF
      } else if (noAct % 5 == 0) {
        neopixelWrite(RGB_BUILTIN,RGB_BRIGHTNESS,RGB_BRIGHTNESS,0); // Yellow
      }
      noAct++;
      setLED = millis();
      digitalWrite(Relay, LOW);
    }
  }
  else if (Mod == ACCESS_GRANTED) {
    neopixelWrite(RGB_BUILTIN,0,RGB_BRIGHTNESS,0); // Green
    digitalWrite(Relay, HIGH);
  } 
  else if (Mod == ACCESS_DENIDED) {
    neopixelWrite(RGB_BUILTIN,RGB_BRIGHTNESS,0,0); // Red
    digitalWrite(Relay, LOW);
  } 
}


void setup()
{
  //Konfigurisanje pinova
  pinMode(Relay, OUTPUT);
  pinMode(Taster, INPUT_PULLUP);
  ledcAttachPin(Buzzer, BUZZER_PWM_CHANNEL);

  Serial.begin(115200);   // Initiate a serial communication
  SPI.begin();      // Initiate  SPI bus
  mfrc522.PCD_Init();   // Initiate MFRC522
  mfrc522.PCD_DumpVersionToSerial();  // Show details of PCD - MFRC522 Card Reader details
  Serial.println("Približite identifikator čitaču...");
  Serial.println();

  //Inicijalizacija nacina signalizacije
  SigMod = NO_ACTION;
}


void loop()
{
  //Ažuriranje signalizacije
  if ( millis() - detectionTime > SIG_DURATION)SigMod = NO_ACTION;
  Signals(SigMod);

  //Zvučni signal
  if (currentNote < notes)
  {
    if(millis()-lastNote > pauseBetweenNotes){
      lastNote=millis();
      int duration = 1000 / noteDur[currentNote];
      tone(Buzzer, melody[currentNote],duration);    
      currentNote++;
      pauseBetweenNotes = duration * 1.30;
    }
  }
  
  // Look for new cards
  if ( !mfrc522.PICC_IsNewCardPresent())
  {
    return;
  }
  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial())
  {
    return;
  }
  //Instructs a PICC in state ACTIVE(*) to go to state HALT.
  mfrc522.PICC_HaltA ();

  detectionTime=millis();

  //Show UID on serial monitor
  Serial.print("UID tag: ");
  for (byte i = 0; i < mfrc522.uid.size; i++)
  {
    content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? "0" : ""));
    content.concat(String(mfrc522.uid.uidByte[i], HEX));
  }
  content.toUpperCase();
  Serial.println(content);

  //Provjera prava pristupa
  Serial.print("Poruka: ");
  if (content.substring(0) == "0D9F6785") //change here the UID of the card/cards that you want to give access
  {
    Serial.println("Pristup dozvoljen");
    SigMod = ACCESS_GRANTED;
    melody=AGmelody, noteDur=AGnoteDurations; notes=sizeof(AGmelody)/sizeof(int);  currentNote=0; pauseBetweenNotes=0; lastNote=0;
  }
  else   {
    Serial.println(" Pristup zabranjen");
    SigMod = ACCESS_DENIDED;
    melody=ADmelody, noteDur=ADnoteDurations; notes=sizeof(ADmelody)/sizeof(int); currentNote=0; pauseBetweenNotes=0; lastNote=0;  
  }
  content="";
}
